from django.conf import settings

PROGRESSBAR_DEFAULTS = {
    'PROGRESSBAR_DEFAULT_TOTAL': 100,
    'PROGRESSBAR_DESTROY_ON_EXIT': False,
    'PROGRESSBAR_DYNAMIC_UPDATE': False
}


class AppSettings:
    """Allowing to override module settings from django config"""

    def __init__(self, defaults):
        for key in defaults:
            if getattr(settings, key, None):
                setattr(self, key, getattr(settings, key))
            else:
                setattr(self, key, defaults[key])


conf = AppSettings(PROGRESSBAR_DEFAULTS)



import pytest
from django.test import TestCase

from django_celery_progressbar.bars import ProgressBar
from django_celery_progressbar.models import TaskProgress


class BarTestCase(TestCase):
    def setUp(self):
        TaskProgress.objects.create(
            task_id='xxx-yyy-zzz',
            step='Brewing coffee',
            total=100
        )

    def test_finalize(self):
        bar = ProgressBar.get('xxx-yyy-zzz')
        bar.finalize()
        assert bar.total == bar.progress, 'progress should be the same as total'

    def test_dynamic_updates(self):
        bar = ProgressBar('xxx-yyy-zzz', _getter=True, dynamic=True)
        task = TaskProgress.objects.get(task_id='xxx-yyy-zzz')
        task.progress = 66
        task.save()

        assert bar.progress == task.progress, 'dynamic update does not work'


class ExceptionTestCase(TestCase):
    def setUp(self):
        TaskProgress.objects.create(
            task_id='xxx-yyy-zzz',
            step='Brewing coffee',
            total=100
        )

    def test_does_not_exist(self):
        with pytest.raises(ProgressBar.exceptions.DoesNotExist):
            ProgressBar.get('asd')

    def test_out_of_range(self):
        with pytest.raises(ProgressBar.exceptions.OutOfRange):
            bar = ProgressBar.get('xxx-yyy-zzz')
            bar.progress += 200

        with pytest.raises(ProgressBar.exceptions.OutOfRange):
            bar = ProgressBar.get('xxx-yyy-zzz')
            bar.update(
                progress=200
            )



class OutOfRange(Exception):
    def __init__(self):
        super(OutOfRange, self).__init__(
            "resulting progress is out of range"
        )


class DoesNotExist(Exception):
    def __init__(self, task_id):
        super(DoesNotExist, self).__init__(
            f"requested progressbar for task {task_id} does not exist"
        )



# Generated by Django 3.0.7 on 2020-06-26 14:20

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TaskProgress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('task_id', models.CharField(max_length=255, unique=True, verbose_name='task_id')),
                ('total', models.IntegerField(default=100)),
                ('progress', models.IntegerField(default=0)),
                ('step', models.CharField(max_length=255, null=True, blank=True)),
            ],
        ),
    ]



from . import exceptions
from .app_settings import conf
from .models import TaskProgress


class ProgressBar:
    """Progressbar manager class"""
    exceptions = exceptions

    def __init__(
            self,
            task_id,
            total=conf.PROGRESSBAR_DEFAULT_TOTAL,
            step=None,
            dynamic=conf.PROGRESSBAR_DYNAMIC_UPDATE,
            _getter=False):
        """
        :param task_id: unique task identifier
        :param total:   maximum progressbar vallue
        :param step:    verbose step name. I.e. 'finding symbolic links'
        :param dynamic: enable dynamic updates
        :param _getter: internal flag
        """
        self.dynamic = dynamic
        self._getter = _getter
        if self._getter:
            try:
                self._db_obj = TaskProgress.objects.get(
                    task_id=task_id
                )
            except TaskProgress.DoesNotExist:
                raise self.exceptions.DoesNotExist(task_id)
        else:
            self._db_obj = TaskProgress(
                task_id=task_id,
                total=total,
                step=step
            )
            self._db_obj.save()

    def __str__(self):
        if self.dynamic:
            self._update_db_obj()
        repr = f"{self._db_obj.progress} / {self._db_obj.total}"
        if self._db_obj.step:
            repr += f" | {self._db_obj.step}"
        return repr

    def __del__(self):
        """delete progressbar model object if configured so"""
        if not self._getter and conf.PROGRESSBAR_DESTROY_ON_EXIT:
            self._db_obj.delete()

    @property
    def progress(self):
        if self.dynamic:
            self._update_db_obj()
        return self._db_obj.progress

    @progress.setter
    def progress(self, value):
        if self._db_obj.progress + value >= self._db_obj.total:
            raise self.exceptions.OutOfRange
        self._db_obj.progress += value
        self._db_obj.save()

    @property
    def step(self):
        if self.dynamic:
            self._update_db_obj()
        return self._db_obj.step

    @step.setter
    def step(self, value):
        self._db_obj.step = value
        self._db_obj.save()

    @property
    def total(self):
        if self.dynamic:
            self._update_db_obj()
        return self._db_obj.total

    @total.setter
    def total(self, value):
        self._db_obj.total = value
        self._db_obj.save()

    @property
    def as_percent(self):
        """return current progress state in percents"""
        if self.dynamic:
            self._update_db_obj()
        return f"{(self._db_obj.progress / self._db_obj.total) * 100}%"

    @classmethod
    def get(cls, task_id):
        """
        get progressbar for a task

        :param task_id: unique task identifier
        :return: ProgressBar object
        """
        return ProgressBar(task_id, _getter=True)

    def update(self, **kwargs):
        """
        update progressbar state

        :param progress: desired bar progress
        :param step: desired bar step
        """
        if kwargs.get('progress'):
            if self._db_obj.progress + kwargs.get('progress') >= self._db_obj.total:
                raise self.exceptions.OutOfRange
            self._db_obj.progress = kwargs['progress']
        if kwargs.get('step'):
            self._db_obj.step = kwargs['step']
        self._db_obj.save()

    def _update_db_obj(self):
        """
        update DB object
        """
        self._db_obj = TaskProgress.objects.get(
            task_id=self._db_obj.task_id
        )

    def finalize(self):
        self._db_obj.progress = self._db_obj.total
        self._db_obj.step = "Complete"
        self._db_obj.save()



from django.db import models


class TaskProgress(models.Model):
    task_id = models.CharField(
        max_length=255,
        unique=True,
        verbose_name='task_id'
    )

    total = models.IntegerField(
        default=100
    )
    progress = models.IntegerField(
        default=0
    )
    step = models.CharField(
        max_length=255,

    )



from pkg_resources import DistributionNotFound, get_distribution

__author__ = 'Eugene Prodan'
__email__ = 'mora9715@gmail.com'

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    pass

default_app_config = 'django_celery_progressbar.apps.ProgressBarConfig'



import setuptools

readme = open("README.rst").read()
history = open("HISTORY.rst").read().replace(".. :changelog:", "")

setuptools.setup(
    name="django_celery_progressbar",
    version="0.2.2",
    author="Eugene Prodan",
    author_email="mora9715@gmail.com",
    description="Progress bar for Django Celery application",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/x-rst",
    url="https://github.com/mora9715/django-celery-progressbar",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers"

    ],
    python_requires='>=3.6',
    install_requires=[
        'django',
        'celery'
    ]
)
